==> graphql-compose-4.3.1.tgz: package/esm/SchemaComposer.js <==
/*  strict */
/* eslint-disable class-methods-use-this */

import { GraphQLObjectType, GraphQLSchema } from './graphql';
import { TypeStorage } from './TypeStorage';
import { TypeMapper } from './TypeMapper';
import { TypeComposer as _TypeComposer } from './TypeComposer';
import { InputTypeComposer as _InputTypeComposer } from './InputTypeComposer';
import { EnumTypeComposer as _EnumTypeComposer } from './EnumTypeComposer';
import { Resolver as _Resolver } from './Resolver';
import { isFunction } from './utils/is';

export class SchemaComposer extends TypeStorage {

  constructor() {
    super();
    const schema = this;

    class TypeComposer extends _TypeComposer {}
    TypeComposer.schemaComposer = schema;
    this.TypeComposer = TypeComposer;

    class InputTypeComposer extends _InputTypeComposer {}
    InputTypeComposer.schemaComposer = schema;
    this.InputTypeComposer = InputTypeComposer;

    class Resolver extends _Resolver {}
    Resolver.schemaComposer = schema;
    this.Resolver = Resolver;

    class EnumTypeComposer extends _EnumTypeComposer {}
    EnumTypeComposer.schemaComposer = schema;
    this.EnumTypeComposer = EnumTypeComposer;

    this.typeMapper = new TypeMapper(schema);

    // alive proper Flow type casting in autosuggestions
    /* :: return this; */
  }

  get Query() {
    return this.rootQuery();
  }

  rootQuery() {
    return this.getOrCreateTC('Query');
  }

  get Mutation() {
    return this.rootMutation();
  }

  rootMutation() {
    return this.getOrCreateTC('Mutation');
  }

  get Subscription() {
    return this.rootSubscription();
  }

  rootSubscription() {
    return this.getOrCreateTC('Subscription');
  }

  buildSchema() {
    const roots = {};

    if (this.has('Query')) {
      const tc = this.getTC('Query');
      this.removeEmptyTypes(tc, new Set());
      roots.query = tc.getType();
    }

    if (this.has('Mutation')) {
      const tc = this.getTC('Mutation');
      this.removeEmptyTypes(tc, new Set());
      roots.mutation = tc.getType();
    }

    if (this.has('Subscription')) {
      const tc = this.getTC('Subscription');
      this.removeEmptyTypes(tc, new Set());
      roots.subscription = tc.getType();
    }

    if (Object.keys(roots).length === 0) {
      throw new Error('Can not build schema. Must be initialized at least one ' + 'of the following types: Query, Mutation, Subscription.');
    }

    return new GraphQLSchema(roots);
  }

  removeEmptyTypes(typeComposer, passedTypes = new Set()) {
    const fieldsOrThunk = typeComposer.getType()._typeConfig.fields;
    const fields = isFunction(fieldsOrThunk) ? fieldsOrThunk() : fieldsOrThunk;
    Object.keys(fields).forEach(fieldName => {
      const fieldType = fields[fieldName].type;
      if (fieldType instanceof GraphQLObjectType) {
        const typeName = fieldType.name;
        if (!passedTypes.has(typeName)) {
          passedTypes.add(typeName);
          const tc = new this.TypeComposer(fieldType);
          if (Object.keys(tc.getFields()).length > 0) {
            this.removeEmptyTypes(tc, passedTypes);
          } else {
            // eslint-disable-next-line
            console.log(`graphql-compose: Delete field '${typeComposer.getTypeName()}.${fieldName}' ` + `with type '${tc.getTypeName()}', cause it does not have fields.`);
            typeComposer.removeField(fieldName);
            delete fields[fieldName];
          }
        }
      }
    });
  }

  getOrCreateTC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _TypeComposer)) {
      const tc = this.TypeComposer.create(typeName);
      this.set(typeName, tc);
      if (onCreate && isFunction(onCreate)) onCreate(tc);
    }
    return this.get(typeName);
  }

  getOrCreateITC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _InputTypeComposer)) {
      const itc = this.InputTypeComposer.create(typeName);
      this.set(typeName, itc);
      if (onCreate && isFunction(onCreate)) onCreate(itc);
    }
    return this.get(typeName);
  }

  getOrCreateETC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _EnumTypeComposer)) {
      const etc = this.EnumTypeComposer.create(typeName);
      this.set(typeName, etc);
      if (onCreate && isFunction(onCreate)) onCreate(etc);
    }
    return this.get(typeName);
  }

  // disable redundant noise in console.logs
  toString() {
    return 'SchemaComposer';
  }
  toJSON() {
    return 'SchemaComposer';
  }
  inspect() {
    return 'SchemaComposer';
  }
}

==> graphql-compose-4.6.0.tgz: package/esm/SchemaComposer.js <==
import _extends from 'babel-runtime/helpers/extends';
/*  strict */
/* eslint-disable class-methods-use-this */

import { GraphQLObjectType, GraphQLSchema } from './graphql';
import { TypeStorage } from './TypeStorage';
import { TypeMapper } from './TypeMapper';
import { TypeComposer as _TypeComposer } from './TypeComposer';
import { InputTypeComposer as _InputTypeComposer } from './InputTypeComposer';
import { EnumTypeComposer as _EnumTypeComposer } from './EnumTypeComposer';
import { InterfaceTypeComposer as _InterfaceTypeComposer } from './InterfaceTypeComposer';
import { Resolver as _Resolver } from './Resolver';
import { isFunction } from './utils/is';
import { getGraphQLType } from './utils/typeHelpers';


export class SchemaComposer extends TypeStorage {

  constructor() {
    super();
    this._schemaMustHaveTypes = [];
    const schema = this;

    class TypeComposer extends _TypeComposer {}
    TypeComposer.schemaComposer = schema;
    this.TypeComposer = TypeComposer;

    class InputTypeComposer extends _InputTypeComposer {}
    InputTypeComposer.schemaComposer = schema;
    this.InputTypeComposer = InputTypeComposer;

    class Resolver extends _Resolver {}
    Resolver.schemaComposer = schema;
    this.Resolver = Resolver;

    class EnumTypeComposer extends _EnumTypeComposer {}
    EnumTypeComposer.schemaComposer = schema;
    this.EnumTypeComposer = EnumTypeComposer;

    class InterfaceTypeComposer extends _InterfaceTypeComposer {}
    InterfaceTypeComposer.schemaComposer = schema;
    this.InterfaceTypeComposer = InterfaceTypeComposer;

    this.typeMapper = new TypeMapper(schema);

    // alive proper Flow type casting in autosuggestions
    /* :: return this; */
  }

  get Query() {
    return this.rootQuery();
  }

  rootQuery() {
    return this.getOrCreateTC('Query');
  }

  get Mutation() {
    return this.rootMutation();
  }

  rootMutation() {
    return this.getOrCreateTC('Mutation');
  }

  get Subscription() {
    return this.rootSubscription();
  }

  rootSubscription() {
    return this.getOrCreateTC('Subscription');
  }

  buildSchema(extraConfig) {
    const roots = {};

    if (this.has('Query')) {
      const tc = this.getTC('Query');
      this.removeEmptyTypes(tc, new Set());
      roots.query = tc.getType();
    }

    if (this.has('Mutation')) {
      const tc = this.getTC('Mutation');
      this.removeEmptyTypes(tc, new Set());
      roots.mutation = tc.getType();
    }

    if (this.has('Subscription')) {
      const tc = this.getTC('Subscription');
      this.removeEmptyTypes(tc, new Set());
      roots.subscription = tc.getType();
    }

    if (Object.keys(roots).length === 0) {
      throw new Error('Can not build schema. Must be initialized at least one ' + 'of the following types: Query, Mutation, Subscription.');
    }

    const types = [...this._schemaMustHaveTypes.map(t => getGraphQLType(t)), // additional types, eg. used in Interfaces
    ...(extraConfig && Array.isArray(extraConfig.types) ? [...extraConfig.types] : [])];

    return new GraphQLSchema(_extends({}, roots, extraConfig, { types }));
  }

  addSchemaMustHaveType(type) {
    this._schemaMustHaveTypes.push(type);
    return this;
  }

  removeEmptyTypes(typeComposer, passedTypes = new Set()) {
    const fieldsOrThunk = typeComposer.getType()._typeConfig.fields;
    const fields = isFunction(fieldsOrThunk) ? fieldsOrThunk() : fieldsOrThunk;
    Object.keys(fields).forEach(fieldName => {
      const fieldType = fields[fieldName].type;
      if (fieldType instanceof GraphQLObjectType) {
        const typeName = fieldType.name;
        if (!passedTypes.has(typeName)) {
          passedTypes.add(typeName);
          const tc = new this.TypeComposer(fieldType);
          if (Object.keys(tc.getFields()).length > 0) {
            this.removeEmptyTypes(tc, passedTypes);
          } else {
            // eslint-disable-next-line
            console.log(`graphql-compose: Delete field '${typeComposer.getTypeName()}.${fieldName}' ` + `with type '${tc.getTypeName()}', cause it does not have fields.`);
            typeComposer.removeField(fieldName);
            delete fields[fieldName];
          }
        }
      }
    });
  }

  getOrCreateTC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _TypeComposer)) {
      const tc = this.TypeComposer.create(typeName);
      this.set(typeName, tc);
      if (onCreate && isFunction(onCreate)) onCreate(tc);
    }
    return this.get(typeName);
  }

  getOrCreateITC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _InputTypeComposer)) {
      const itc = this.InputTypeComposer.create(typeName);
      this.set(typeName, itc);
      if (onCreate && isFunction(onCreate)) onCreate(itc);
    }
    return this.get(typeName);
  }

  getOrCreateETC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _EnumTypeComposer)) {
      const etc = this.EnumTypeComposer.create(typeName);
      this.set(typeName, etc);
      if (onCreate && isFunction(onCreate)) onCreate(etc);
    }
    return this.get(typeName);
  }

  getOrCreateIFTC(typeName, onCreate) {
    if (!this.hasInstance(typeName, _InterfaceTypeComposer)) {
      const iftc = this.InterfaceTypeComposer.create(typeName);
      this.set(typeName, iftc);
      if (onCreate && isFunction(onCreate)) onCreate(iftc);
    }
    return this.get(typeName);
  }

  // disable redundant noise in console.logs
  toString() {
    return 'SchemaComposer';
  }

  toJSON() {
    return 'SchemaComposer';
  }

  inspect() {
    return 'SchemaComposer';
  }

  clear() {
    super.clear();
    this._schemaMustHaveTypes = [];
  }
}
